void setup_gcode() {
  ////// GCODE FEEDER
  DebugSerial.begin(115200);  // regular serial monitor for troubleshooting
  GrblSerial.begin(115200, SERIAL_8N1, 16, 17); // // streams gcode to the controller
  GrblSerial.setRxBufferSize(1024);
  GrblSerial.flush();
  pinMode (16, INPUT_PULLUP ); // added to force a level when serial wire is not connected
  while (GrblSerial.available() > 0) GrblSerial.read() ; // clear the serial2 buffer
  delay(200);    // wait that all char are sent
  currentPattern = 0;         // initialize the pattern array index - later incremented in openFileSD()
 
    checkSD();                  // make sure SD is working
    delay(2000);                // needed this or would sometimes miss the homing cycle
    root = SD.open("/"); // opening to count files
    countFiles(root); // counting files and saving as fileCount
    randomizePatterns();        // fills the patternOrder[] array and scrambles the index

}


void nextPattern() {
  currentPattern++;
  if (currentPattern >= fileCount) {
    // start over when we reach the end
    randomizePatterns();
    //    currentPattern = 0;
  }
}

void checkSD() {

  // Check if SD card is OK

  // On the Ethernet Shield, CS is pin 4. It's set as an output by default.
  // Note that even if it's not used as the CS pin, the hardware SS pin
  // (10 on most Arduino boards, 53 on the Mega) must be left as an output
  // or the SD library functions will not work.
  // If possible, it's easiest to just use the SS pin for CS.
  // pinMode(10,OUTPUT); //if you use a different pin for CS, you might need these two lines
  // digitalWrite(10,HIGH); //change the pin number to 53 for a mega

  while (!SD.begin(5)) {
    DebugSerial.println("Please insert SD card...\n");
    //  GrblSerial.println("Please insert SD card...\n");
    delay(1000);
    SD_present = false;
  }
  DebugSerial.println("SD card OK...\n");
  SD_present = true;
  delay(1000);
}

// Counts files to be used in selecting from the array
void countFiles(File& dir) {
  fileCount = 0;
  while (true) {
    File entry =  dir.openNextFile();
    if (! entry) {
      // no more files
      break;
    }

    String upper_filename(entry.name());
    upper_filename.toUpperCase();
    if (entry.isDirectory()) {
      // Do nothing with directories
    } else if (upper_filename.startsWith("/HOME")) {
      // Store this name, and move on.
      homeFilename = entry.name();
      DebugSerial.print("Found home ");
      DebugSerial.print(entry.name());
      DebugSerial.println("\tsz: " + String(entry.size() / 1024) + "kB");
    } else {
      // it's a file, add to the file count
      filenames[fileCount] = entry.name();
      DebugSerial.print("Found pattern ");
      DebugSerial.print(entry.name());
      DebugSerial.println("\tsz: " + String(entry.size() ) + "Bytes");
      fileCount++;
      if (fileCount >= MAX_FILES) {
        DebugSerial.println("ERROR: Too many patterns! Overwriting the last one.");
        // Go back one index. This will end in problems, but it will be more stable.
        fileCount--;
      }
    }
    entry.close();
  }
  DebugSerial.println("File count is " + String(fileCount)); //this shows up in the serial monitor if hooked up to a computer
}


void homeTable() {
  if (homeFilename.length() == 0) {
    // There's no home file on the disk, don't do anything
    DebugSerial.println("No home.GCODE file found.");
    return;
  }

  currentFile = SD.open(homeFilename, FILE_READ);
  DebugSerial.print("-- ");
  DebugSerial.print("File : ");
  DebugSerial.print(homeFilename);
  DebugSerial.print(" opened!");
  DebugSerial.println(" --\n");
  DebugSerial.println("StartGcode Home");
  startGcode();
  while (!endFile) {
    sendGcode();
    delay(10);
  }
  endGcode();
  DebugSerial.println("EndGcode Home");
}

// Randomize the indices. This will leave exactly one instance of each number in the array, and it
// will leave them in some random order.
void randomizePatterns() {

  //first fill array with sequential numbers
  for (int i = 0; i < MAX_FILES; ++i) {
    patternOrder[i] = i;
  }

  randomSeed(esp_random()); // this seeds the random number generator

  // randomize the array. swap each item once with a new location.
  for (int j = 0; j < fileCount; j++) {
    int pos = random(fileCount);

    // swap pos and j
    int t = patternOrder[j];
    patternOrder[j] = patternOrder[pos];
    patternOrder[pos] = t;
  }
  currentPattern = random(fileCount);
}


void openFileSD() {

  // Fetch the next pattern id
  int nextPatternNumber = patternOrder[currentPattern];
  String fileName = filenames[nextPatternNumber];

  // open the file for printing
  currentFile = SD.open(fileName, FILE_READ); //VERIFY!!
  DebugSerial.print("-- ");
  DebugSerial.print("File : ");
  DebugSerial.print(fileName);
  DebugSerial.print(" opened!");
  DebugSerial.println(" --\n");
  delay(1000);
}

void emptySerialBuf() {
  while (GrblSerial.available()) {
    GrblSerial.read();
  }
}

// Wait for data on Serial
// Argument serialNum for Serial number
void waitSerial() {
  while (!GrblSerial.available()) {
    // checkButton(); // Check the button while we are waiting
    delay(1);
  }
}

// Return String  from serial line reading
String getSerial() {

  waitSerial();

  String inLine = "";
  while (GrblSerial.available()) {
    inLine += (char)GrblSerial.read();
    // checkButton(); // Check the button while we are waiting
    delay(2);
  }
  return inLine;
}

// Checks the button, and determines if there was a button press.
// OK, this might be a little confusing, so here's how it (is supposed to) work.
// This function needs to be checked frequently, as often as possible, almost.
// This function will read the state of the button. If it doesn't match our current state, then it will
// add the time since the last check to the counter. If the counter goes over DEBOUNCE_TIME_MS, then
// The state change happens.
//
// If the state change goes from false to true, then we just pressed the button.
// This debounce is two way, so that if we have the switch in place, and it registers a false
// momentarily, it won't also trigger an abort.
//
/*
  void checkButton() {
  // Static variables (local, but persistent).
  static int debounceCounter = 0;
  static unsigned long prevTime = millis();

  // Since we are triggering on a false to true, set this to true to avoid the first trigger.
  static boolean buttonState = true;

  // Save the dt right now.
  unsigned long deltaTime_ms = millis() - prevTime;
  // Set up for the next dt.
  prevTime += deltaTime_ms;

  // Read the button
  boolean buttonPressed = (LOW == digitalRead(CONTINUE_PIN));

  if ((buttonPressed != buttonState) && debounceCounter < DEBOUNCE_TIME_MS) {
    // The button is changing, but we haven't triggered the change state yet.
    debounceCounter += deltaTime_ms;

    if (debounceCounter >= DEBOUNCE_TIME_MS) {
      // The state of the button has changed.

      if (buttonPressed) {
        // We just pressed the button.
        abortPattern = true;
      }

      buttonState = buttonPressed;
    }
  }

  if (buttonPressed == buttonState) {
    // We stopped pushing the button
    debounceCounter = 0;
  }
  }
*/

void startGcode() {
  GrblSerial.print("\r\n\r\n");          //Wake up grbl
  delay(2);
  emptySerialBuf();
  startFile = true;
  endFile = false;
  nextRow = true;
}


void sendGcode() {
  // READING GCODE FILE AND SEND ON SERIAL PORT TO GRBL
  // START GCODE SENDING PROTOCOL ON SERIAL 1
  String line = "";

  if (currentFile) {
    // until the file's end
    if (currentFile.available()) {
      /*
            checkButton();
            if (abortPattern) {
              // The user pressed the button!
              DebugSerial.println("Pattern Aborted");
              break;
            }
      */
      line = readLine(currentFile);        // read line in gcode file
      if (line.length() > 1) {
        nextRow = false;
        DebugSerial.print(line);        // send to serials
        GrblSerial.print(line);
      }

    }
    else {
      endFile = true;
      startFile = false;
    }
  }
  else {
    fileError();
  }
}

void endGcode() {
  currentFile.close();
  DebugSerial.println("Finish!!\n");
  delay(2000);

}

/*
  void sendGcode() {
    // Testing Code - comment out the sendGcode stuff above and uncomment this
    // to test without having to stream the whole gcode file

    DebugSerial.println("Printing the file.");
    currentFile.close();
    DebugSerial.println("Finish!\n");
    delay(2000);
    // End Testing code
  }
*/

// For file open or read error
void fileError() {

  DebugSerial.println("\n");
  DebugSerial.println("File Error !");
  startFile = false;
  endFile = true;
}

// return line from file reading
String readLine(File f) {

  char inChar = 0;
  String line = "";

  do {
    inChar = (char)f.read();
    line += inChar;
  } while (inChar != '\n');

  return line;
}

////////////////////////
/*
  void serialEvent() {
  String inLine = "";
  while (GrblSerial.available()) {
    // get the new byte:
    char inChar = (char)GrblSerial.read();
    // add it to the inputString:
    inLine += inChar;
    // if the incoming character is a newline, set a flag so the main loop can
    // do something about it:
    if (inChar == '\n') {
      nextRow = true;
    }
    delay(1);
  }
   DebugSerial.print(inLine); // print grbl return on serial
  }
*/


////////////////////////////////
void readGrbl() {
  String inLine = "";
  while (GrblSerial.available()) {
    // get the new byte:
    char inChar = (char)GrblSerial.read();
    // add it to the inputString:
    inLine += inChar;
    // if the incoming character is a newline, set a flag so the main loop can
    // do something about it:
    if (inChar == '\n') {
      nextRow = true;
    }
    delay(1);
  }
  if (inLine.length() > 0)
    DebugSerial.print(inLine); // print grbl return on serial
}
